//////////////////////////////////////////////
/////       Variables de gráfico        //////
//////////////////////////////////////////////


//Dimensiones del gráfico
var width = 960,
  height = 450,
  radius = Math.min(width, height) / 2;

//Márgenes del gráfico
 var margin_x = 0;
 var margin_y = 50;

//Elemento SGV que va a soportar nuestro grafico
var svg = d3.select("body").append("svg")
  .attr("id", "grafico")
  .attr("width", width)
  .attr("height", height)
  .append("g")
  .attr("transform", "translate(" + width / 2 + "," + height / 2 + ")");

//Añadir los sectores, líneas y labels
svg.append("g")
  .attr("class", "slices");
svg.append("g")
  .attr("class", "labels");
svg.append("g")
  .attr("class", "lines");

//Añadir el título del gráfico (esta fuera del elemento SVG)
    var title = d3.select("body")
        .append("div")
        .attr("class", "title")
        .style("left",(-150 + margin_x + width/2 )+ "px")
        .style("top", (margin_y - 25) + "px");


    title.append("text")
        .text("Mortalidad infantil por zona y año");

//Seleccion de la gama de colores
    var color = d3.scale.ordinal()
    .domain(["Pacifico_y_Europa_del_Este","Europa_y_Asia_Central","Latino_America_y_el_Caribe","Oriente_Medio_y_Africa_del_Norte","Sur_de_Asia","Africa_Sub-Sahariana"])
    .range(["#98abc5", "#8a89a6", "#7b6888", "#6b486b", "#a05d56", "#d0743c"])

//Pintar el texto de los checkboxes del color correspondiente a su continente
//a modo de leyenda
    d3.select(".Pacifico_y_Europa_del_Este").style("color","#98abc5");
    d3.select(".Europa_y_Asia_Central").style("color","#8a89a6");
    d3.select(".Latino_America_y_el_Caribe").style("color","#7b6888");
    d3.select(".Oriente_Medio_y_Africa_del_Norte").style("color","#6b486b");
    d3.select(".Sur_de_Asia").style("color","#a05d56");
    d3.select(".Africa_Sub-Sahariana").style("color","#d0743c");


//Variables de los arcos exteriores e interiores que forman el gráfico
var arc = d3.svg.arc()
  .outerRadius(radius * 0.8)
  .innerRadius(radius * 0.4);

var outerArc = d3.svg.arc()
  .innerRadius(radius * 0.9)
  .outerRadius(radius * 0.9);

//Año inicial
var anyo = 2015;



//////////////////////////////////////////////////
/////Construcción e inicialización de objetos/////
//////////////////////////////////////////////////


//Encontrar los radio buttons de la página
var radios = document.querySelectorAll('input[type=radio][name="anyo"]');

//Seleccionar el año actual a representar
radios.forEach(function (d) {
  if (parseInt(d.id) == anyo) {
    d.checked = true;
  }
});

//Encontrar los checkboxes de la página
var asia = document.querySelector("input[name=Pacifico_y_Europa_del_Este]");
var europa = document.querySelector("input[name=Europa_y_Asia_Central]");
var america = document.querySelector("input[name=Latino_America_y_el_Caribe]");
var norte_africa = document.querySelector("input[name=Oriente_Medio_y_Africa_del_Norte]");
var sur_asia = document.querySelector("input[name=Sur_de_Asia]");
var africa_sur = document.querySelector("input[name=Africa_Sub-Sahariana]");

//Seleccionar todos los checkboxes
asia.checked = true;
europa.checked = true;
america.checked = true;
norte_africa.checked = true;
sur_asia.checked = true;
africa_sur.checked = true;

//Crear un array que indica las zonas que se deben mostrar.
//Inicialmente se dibujan todos
var continentes = [
  { continente: "Pacifico_y_Europa_del_Este", dibujar: asia.checked },
  { continente: "Europa_y_Asia_Central", dibujar: europa.checked },
  { continente: "Latino_America_y_el_Caribe", dibujar: america.checked },
  { continente: "Oriente_Medio_y_Africa_del_Norte", dibujar: norte_africa.checked },
  { continente: "Sur_de_Asia", dibujar: sur_asia.checked },
  { continente: "Africa_Sub-Sahariana", dibujar: africa_sur.checked },
]



//////////////////////////////////////////////////
/////       Lectura de los datos            //////
//////////////////////////////////////////////////


d3.csv("data1.csv", function (error, data) {

  //Crear una variable key con los nombres de las zonas
  var key = function(d){ return d.data.names; };

  //Registrar manejadores de eventos para los radio buttons
  radios.forEach(function (d) {
    d.addEventListener('change', cambiarAnyo);
  });

  //Función que recibe el evento de selección de otro año y desencadena
  //la modificación del gráfico
  function cambiarAnyo(event) {
    anyo = parseInt(this.value);
    modificar_grafico();
  }

  //Registrar manejadores de eventos para los checkboxes
  asia.addEventListener('change', cambiarContinente);
  europa.addEventListener('change', cambiarContinente);
  america.addEventListener('change', cambiarContinente);
  norte_africa.addEventListener('change', cambiarContinente);
  sur_asia.addEventListener('change', cambiarContinente);
  africa_sur.addEventListener('change', cambiarContinente);

  //Función que recibe el evento de alguna modificación en los continentes
  //y desencadena la modificación del gráfico
  function cambiarContinente(event) {
    continentes[0].dibujar = asia.checked;
    continentes[1].dibujar = europa.checked;
    continentes[2].dibujar = america.checked;
    continentes[3].dibujar = norte_africa.checked;
    continentes[4].dibujar = sur_asia.checked;
    continentes[5].dibujar = africa_sur.checked;
    modificar_grafico();
  }

  //Guardar información sobre las opciones seleccionadas y filtrar datos
  var dataset = filtrarDatos();

  //Añadir los datos iniciales
  var pie = d3.layout.pie()
  .sort(null)
  .value(function (d) { return d.x; });

  //Crea el gráfico inicial
  modificar_grafico();

  //Función para seleccionar los datos. Se almacenan en una variable dataset.
  function filtrarDatos() {

    var dataset2;

    //Se selecciona el año, se modifica la columna de x
    //en función de ese año
    if (anyo == 2015) {
      data.forEach(function (d) {
        d.x = parseInt(d.rate2015);
      });
    dataset2 = data.filter(function(d){return (d.rate2015 != "NA")});
    }

    if (anyo == 2010) {
      data.forEach(function (d) {
        d.x = parseInt(d.rate2010);
      });
    dataset2 = data.filter(function(d){return (d.rate2010 != "NA")});
    }

    if (anyo == 2005) {
      data.forEach(function (d) {
        d.x = parseInt(d.rate2005);
      });
    dataset2 = data.filter(function(d){return (d.rate2005 != "NA")});
    }

    if (anyo == 2000) {
      data.forEach(function (d) {
        d.x = parseInt(d.rate2000);
      });
    dataset2 = data.filter(function(d){return (d.rate2000 != "NA")});
    }

    if (anyo == 1995) {
      data.forEach(function (d) {
        d.x = parseInt(d.rate1995);
      });
    dataset2 = data.filter(function(d){return (d.rate1995 != "NA")});
    }

    if (anyo == 1990) {
      data.forEach(function (d) {
        d.x = parseInt(d.rate1990);
      });
    dataset2 = data.filter(function(d){return (d.rate1990 != "NA")});
    }

    //Selecciona las zonas a representar
    continentes.forEach(function (d) {
      if (d.dibujar == false) {
        dataset2 = dataset2.filter(function (m) { return (m.names != d.continente) });
      }
    });
    //console.log(dataset2);
    return dataset2;

  }

  //Función que modifica el gráfico cuando se ha cambiado de zona o de año
  function modificar_grafico (){

    dataset = filtrarDatos();

    //Creación de los sectores con transiciones
    var slice = svg.select(".slices").selectAll("path.slice")
        .data(pie(dataset),key);

    slice.enter()
        .insert("path")
        .style("fill", function(d){return color(d.data.names);})
        .attr("class", "slice");

    slice
        .transition().duration(1000)
        .attrTween("d", function(d) {
            this._current = this._current || d;
            var interpolate = d3.interpolate(this._current, d);
            this._current = interpolate(0);
            return function(t) {
                return arc(interpolate(t));
            };
        })

    slice.exit()
        .remove();

    //Interacción cuando se posiciona el cursor sobre un sector
    slice.on("mouseenter", function (d) {
            d3.select(this)
                .attr("stroke", function (d) {
                return color(d.data.names);})
                .transition()
                .duration(500)
                .attr("d", outerArc)
                .attr("stroke-width", 1);

      })
         .on("mouseleave", function (d) {
             d3.select(this).transition()
                 .duration(500)
                 .attr("d", arc)
                 .attr("stroke", "none");
         })


    //Creación de los labels con transiciones
    var text = svg.select(".labels").selectAll("text")
        .data(pie(dataset), key);

    text.enter()
        .append("text")
        .attr("dy", ".25em");/*
        .style("fill", function(d){return color(d.data.names);})
        .text(function(d) {
          console.log("Primera vez" + anyo);
          if (anyo == 2015)
            return d.data.rate1990;
        });*/
        text.attr("dy", ".25em")
            .style("fill", function(d){return color(d.data.names);})
            .text(function(d) {
              //console.log("segunda vez" + anyo);
              if(anyo == 2015) {
                return d.data.rate2015+"%";}
              if(anyo == 2010) {
                return d.data.rate2010+"%";}
              if(anyo == 2005) {
                return d.data.rate2005+"%";}
              if(anyo == 2000) {
                return d.data.rate2000+"%";}
              if(anyo == 1995) {
                return d.data.rate1995+"%";}
              if(anyo == 1990) {
                return d.data.rate1990+"%";}
            });

    function midAngle(d){
        return d.startAngle + (d.endAngle - d.startAngle)/2;
    }

    text.transition().duration(1000)
        .attrTween("transform", function(d) {
            this._current = this._current || d;
            var interpolate = d3.interpolate(this._current, d);
            this._current = interpolate(0);
            return function(t) {
                var d2 = interpolate(t);
                var pos = outerArc.centroid(d2);
                pos[0] = radius * (midAngle(d2) < Math.PI ? 1 : -1);
                return "translate("+ pos +")";
            };
        })
        .styleTween("text-anchor", function(d){
            this._current = this._current || d;
            var interpolate = d3.interpolate(this._current, d);
            this._current = interpolate(0);
            return function(t) {
                var d2 = interpolate(t);
                return midAngle(d2) < Math.PI ? "start":"end";
            };
        });

    text.exit()
        .remove();

    //Creación de las líneas con transiciones
    var polyline = svg.select(".lines").selectAll("polyline")
        .data(pie(dataset), key);

    polyline.enter()
        .append("polyline");

    polyline.transition().duration(1000)
        .attrTween("points", function(d){
            this._current = this._current || d;
            var interpolate = d3.interpolate(this._current, d);
            this._current = interpolate(0);
            return function(t) {
                var d2 = interpolate(t);
                var pos = outerArc.centroid(d2);
                pos[0] = radius * 0.95 * (midAngle(d2) < Math.PI ? 1 : -1);
                return [arc.centroid(d2), outerArc.centroid(d2), pos];
            };
        });

    polyline.exit()
        .remove();
    };
});
